﻿using System;
using System.ComponentModel.Composition;
using Caliburn.Micro;

namespace CardPlay.Client.Tasks
{
    public class OpenDialog<TDialog> : ITask where TDialog : class, IDialog
    {
        public OpenDialog(TDialog dialog)
        {
            Dialog = dialog;
        }

        public TDialog Dialog { get; private set; }

#if ! WINDOWS_PHONE
        [Import]
        public IWindowManager WindowManager { get; set; }
#endif

        public event EventHandler<ResultCompletionEventArgs> Completed = delegate { };

        public void Execute(ActionExecutionContext context)
        {
            Dialog.Completed += OnDialogCompleted;
#if ! WINDOWS_PHONE
            WindowManager.ShowDialog(Dialog);
#endif
        }

        private void OnDialogCompleted(object sender, DialogResultCompletionEventArgs e)
        {
            var screen = sender as IScreen;
            if (screen == null) throw new ArgumentException("sender");

            screen.CanClose(delegate(bool a) { if (!a) return; });
            screen.TryClose();

            Dialog.Completed -= OnDialogCompleted;

            var args = new ResultCompletionEventArgs
                       {
                           WasCancelled = !(e.DialogResult.HasValue && e.DialogResult.Value),
                           Error = e.Error
                       };

            Completed(this, args);
        }
    }
}