﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

namespace CardPlay.Model.Layout
{
    public abstract class CompositeLayout : LayoutArea, IEnumerable 
    {
        private List<ILayoutSite> _children;

        #region ILayoutZone Members

        //public override void Measure(ILayoutContext context, Size availableSize)
        //{
        //    base.Measure(context, availableSize);

        //    foreach (var layout in Children)
        //        layout.Measure(context, availableSize);

        //    var measurements = OnMeasure(context, availableSize);

        //    MeasuredSize.MinimumSize = ConstrainedSize(measurements.MinimumSize);
        //    MeasuredSize.DesiredSize = ConstrainedSize(measurements.DesiredSize);
        //}

        //public override void Arrange(ILayoutContext context, Rectangle bounds)
        //{
        //    base.Arrange(context, bounds);
        //    OnArrange(context, Bounds);
        //}

        #endregion

        public HorizontalAlignment HorizontalContentAlignment
        {
            get;
            set;
        }

        public VerticalAlignment VerticalContentAlignment
        {
            get;
            set;
        }

        public int Count
        {
            get { return Children.Count; }
        }

        public void Add(ILayoutSite item)
        {
            Guard.Against<ArgumentNullException>(item == null, "item");
            Guard.Against<InvalidOperationException>(Children.Contains(item), "item already exists in the layout");
                
            Children.Add(item);
// ReSharper disable PossibleNullReferenceException
            item.Parent = this;
// ReSharper restore PossibleNullReferenceException
        }

        public void Remove(ILayoutSite item)
        {
            Guard.Against<ArgumentNullException>(item == null, "item");
            Guard.Against<InvalidOperationException>(! Children.Contains(item), "item does not exist in the layout");

// ReSharper disable PossibleNullReferenceException
            item.Parent = null;
// ReSharper restore PossibleNullReferenceException
            Children.Remove(item);
        }

        public IList<ILayoutSite> VisibleChildren
        {
            get { return Children.Where(item => !item.Collapsed).ToArray(); }
        }

        public IList<ILayoutSite> Children
        {
            get { return _children ?? (_children = new List<ILayoutSite>()); }
        }

        //protected abstract Size OnMeasure(ILayoutContext context, Size availableSize);
        //protected abstract void OnArrange(ILayoutContext context);

        #region IEnumerable Members

        IEnumerator IEnumerable.GetEnumerator()
        {
            return Children.GetEnumerator();
        }

        #endregion
    }
}
