﻿using System;
using CardPlay.Model.Primitives;

namespace CardPlay.Model.Layout
{
    public static class ExtensionsToSize
    {
        public static Size ConstrainedTo(this Size size, Size minSize, Size maxSize)
        {
            return new Size(
                Math.Min(Math.Max(minSize.Width, size.Width), maxSize.Width),
                Math.Min(Math.Max(minSize.Height, size.Height), maxSize.Height));
        }

        public static Size BelowUpperLimit(this Size size, Size upperLimit)
        {
            return new Size(
                Math.Min(upperLimit.Width, size.Width),
                Math.Min(upperLimit.Height, size.Height));
        }

        public static Size BeyondLowerLimit(this Size size, Size lowerLimit)
        {
            return new Size(
                Math.Max(lowerLimit.Width, size.Width),
                Math.Max(lowerLimit.Height, size.Height));
        }

        public static Rectangle PositionWithin(this Size size, Rectangle bounds, HorizontalAlignment horizontalAlignment, VerticalAlignment verticalAlignment)
        {
            var location = bounds.Location;

            switch (horizontalAlignment)
            {
                case HorizontalAlignment.Left:
                    break;

                case HorizontalAlignment.Right:
                    if (size.Width < bounds.Width)
                        location.X = bounds.Right - size.Width;
                    break;

                case HorizontalAlignment.Center:
                    if (size.Width < bounds.Width)
                        location.X += (bounds.Width - size.Width) / 2;
                    break;

                case HorizontalAlignment.Stretch:
                    size.Width = Math.Max(size.Width, bounds.Width);
                    break;
            }

            switch (verticalAlignment)
            {
                case VerticalAlignment.Top:
                    break;

                case VerticalAlignment.Bottom:
                    if (size.Height < bounds.Bottom)
                        location.Y = bounds.Bottom - size.Height;
                    break;

                case VerticalAlignment.Center:
                    if (size.Height < bounds.Height)
                        location.Y += (bounds.Height - size.Height) / 2;
                    break;

                case VerticalAlignment.Stretch:
                    size.Height = Math.Max(size.Height, bounds.Height);
                    break;
            }

            return new Rectangle(location, size);
        }
    }
}