﻿using System;
using System.Collections.Generic;
using System.Diagnostics.Contracts;
using System.Linq;

namespace CardPlay.Model.Layout
{
    public class GridLayoutBase : CompositeLayout
    {
        private int _cols;
        private int _rows;

        public int Columns
        {
            get { return _cols; }
            set
            {
                Contract.Requires<ArgumentOutOfRangeException>(value >= 0, "value");
                _cols = value;
            }
        }

        public int Rows
        {
            get { return _rows; }

            set
            {
                Contract.Requires<ArgumentOutOfRangeException>(value >= 0, "value");
                _rows = value;
            }
        }

        protected IEnumerable<ILayoutSite> SitesInColumn(int column)
        {
            var end = Math.Min(Columns * Rows, Count);

            for (var i = column; i < end; i += Columns)
                yield return Children[i];
        }

        protected IEnumerable<ILayoutSite> SitesInRow(int row)
        {
            return Children.Skip(Columns * row).Take(Columns);
        }

        protected int[] HorizontalAxisLayout
        {
            get
            {
                var layout = new int[Columns];

                for (var row = 0; row < Rows; ++row)
                {
                    var i = 0;

                    foreach (var size in layout.Zip(SitesInRow(row), (width, site) => site.Collapsed ? 0 : Math.Max(width, site.MeasuredSize.Width)))
                        layout[i++] = size;
                }

                return layout;
            }
        }

        protected int[] VerticalAxisLayout
        {
            get
            {
                var layout = new int[Rows];

                for (var col = 0; col < Columns; ++col)
                {
                    var i = 0;

                    foreach (var size in layout.Zip(SitesInColumn(col), (height, site) => site.Collapsed ? 0 : Math.Max(height, site.MeasuredSize.Height)))
                        layout[i++] = size;
                }

                return layout;
            }
        }
    }
}