﻿using CardPlay.Model.Primitives;

namespace CardPlay.Model.Layout
{
    public class LayoutArea : ILayoutSite
    {
        private Rectangle _bounds = Rectangle.Empty;
        private MarginSize _margin = MarginSize.Empty;

        public LayoutArea()
        {
            MaximumSize = new Size(int.MaxValue, int.MaxValue);
        }

        public MarginSize Margin
        {
            get { return _margin; }
            set { _margin = value; }
        }

        #region ILayoutArea Members

        public ILayoutSite Parent { get; set; }

        public Rectangle Bounds
        {
            get { return _bounds; }
            set { _bounds = value; }
        }

        public Size MeasuredSize
        {
            get; protected set;
        }

        public Size MinimumSize
        {
            get;
            set;
        }

        public Size MaximumSize
        {
            get;
            set;
        }

        public void Measure(ILayoutContext context, Size availableSize)
        {
            if (Collapsed) return;

            var contentSize = (availableSize - Margin.Size).BeyondLowerLimit(Size.Empty);
            MeasuredSize = (OnMeasure(context, contentSize) + Margin.Size).ConstrainedTo(MinimumSize, MaximumSize);
        }

        public void Arrange(ILayoutContext context, Rectangle bounds)
        {
            if (Collapsed) return;

            Bounds = bounds;

            var contentBounds = Bounds;

            contentBounds.Size = (Bounds.Size - Margin.Size).BeyondLowerLimit(Size.Empty);
            contentBounds.Offset(Margin.Left, Margin.Top);

            OnArrange(context, contentBounds);
        }

        protected virtual Size OnMeasure(ILayoutContext context, Size availableSize)
        {
            return Size.Empty;
        }

        protected virtual void OnArrange(ILayoutContext context, Rectangle bounds)
        {
        }

        #endregion

        public Point Location
        {
            get { return Bounds.Location; }
            set { _bounds.Location = value; }
        }

        public Size Size
        {
            get { return Bounds.Size; }
            set { _bounds.Size = value; }
        }

        public bool Collapsed
        {
            get; set;
        }
    }
}
