﻿using System.Linq;
using CardPlay.Model.Primitives;

namespace CardPlay.Model.Layout
{
    public class UniformGridLayout : GridLayoutBase
    {
        protected override Size OnMeasure(ILayoutContext context, Size availableSize)
        {
            var cellSize = CalcCellSize(availableSize);

            foreach (var visibleChild in VisibleChildren)
                visibleChild.Measure(context, cellSize);

            var maximumCellSize = VisibleChildren.Aggregate(Size.Empty, (result, site) => result.BeyondLowerLimit(site.MeasuredSize));

            return new Size(Columns * maximumCellSize.Width, Rows * maximumCellSize.Height);
        }

       protected override void OnArrange(ILayoutContext context, Rectangle bounds)
       {
            var cellSize = CalcCellSize(bounds.Size);
            var origin = new Size(Columns * cellSize.Width, Rows * cellSize.Height)
                .PositionWithin(bounds, HorizontalContentAlignment, VerticalContentAlignment)
                .Location;
        
            var col = 0;
            var row = 0;
        
            foreach (var layout in Children)
            {
                if (! layout.Collapsed)
                    layout.Arrange(context, new Rectangle(origin + new Size(col * cellSize.Width, row * cellSize.Height), cellSize));
                
                if (++col == Columns)
                {
                    col = 0;
                    row++;
                }
            }
        }

        private Size CalcCellSize(Size layoutSize)
        {
            return new Size(Columns > 0 ? layoutSize.Width / Columns : 0, Rows > 0 ? layoutSize.Height / Rows : 0);
        }
    }
}
